import 'dart:convert';

import 'package:ocg_app/ocg_app.dart';
import 'package:http/http.dart' as http;
import 'package:uuid/uuid.dart';

import '../models/user.dart';

abstract class UserRepository {
  Future<AppResponse<User>> getSelf();
  Future<AppResponse<String>> getSelfId();
  Future<AppResponse<User>> createUser(User user);
  Future<AppResponse<User>> getUser(String userId);
  Future<AppResponse<List<User>>> getAllUsers();
}

class UserTestRepository extends UserRepository {
  @override
  Future<AppResponse<User>> createUser(User user) async {
    return AppResponse.named(
        data: User.named(
            id: "1",
            name: "Bhavneet singh",
            status: "Available",
            profilePic:
                "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4"));
  }

  @override
  Future<AppResponse<List<User>>> getAllUsers() async {
    return AppResponse.named(data: [
      User.named(
          id: "1",
          name: "Bhavneet singh",
          status: "Available",
          profilePic:
              "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4")
    ]);
  }

  @override
  Future<AppResponse<User>> getUser(String userId) async {
    return AppResponse.named(
        data: User.named(
            id: "1",
            name: "Bhavneet singh",
            status: "Available",
            profilePic:
                "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4"));
  }

  @override
  Future<AppResponse<User>> getSelf() async {
    return AppResponse.named(
        data: User.named(
            id: "1",
            name: "Bhavneet singh",
            status: "Available",
            profilePic:
                "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4"));
  }

  @override
  Future<AppResponse<String>> getSelfId() async {
    return AppResponse.named(data: "1");
  }
}

class UserNetworkRepository extends UserRepository {
  final TokenRepository tokenRepository;
  final http.BaseClient client;

  UserNetworkRepository(this.tokenRepository, this.client);

  @override
  Future<AppResponse<User>> createUser(User user) {
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<List<User>>> getAllUsers() async {
    return AppResponse.named(data: [
      User.named(
          id: Uuid().v4().substring(0, 24),
          name: "Bhavneet singh1",
          status: "Available",
          profilePic:
              "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4")
    ]);
  }

  @override
  Future<AppResponse<User>> getSelf() async {
    try {
      // String url = "${ApiConfig.BASE_URL}/tmp/user/";
      // var response = await client.get(url);
      // var json = jsonDecode(response.body);
      return AppResponse.named(
          data: User.named(
              id: "1",
              name: "Bhavneet singh",
              status: "Available",
              profilePic:
                  "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4"));
    } catch (e) {
      return AppResponse.named(error: "${e.toString()}");
    }
  }

  @override
  Future<AppResponse<String>> getSelfId() async {
    AppResponse<AppToken> response = await tokenRepository.getToken();
    if (response.isSuccess) {
      return AppResponse.named(data: response.data.userId);
    }
    return AppResponse.named(error: response.error);
  }

  @override
  Future<AppResponse<User>> getUser(String userId) async {
    return AppResponse.named(
        data: User.named(
            id: "1",
            name: "Bhavneet singh",
            status: "Available",
            profilePic:
                "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4"));
  }
}
